/*-----------------------------------------------------------------------

File  : che_to_weightgen.h

Author: Stephan Schulz

Contents
 
  Routines for generating weights for term orderings

Changes

<1> Fri Sep 25 02:49:11 MET DST 1998
    New

-----------------------------------------------------------------------*/

#ifndef CHE_TO_WEIGHTGEN

#define CHE_TO_WEIGHTGEN


#include <che_clausesetfeatures.h>
#include <clb_simple_stuff.h>

/*---------------------------------------------------------------------*/
/*                    Data type declarations                           */
/*---------------------------------------------------------------------*/

typedef enum
{
   WNoMethod = 0,       /* Nothing */
   WConstantWeight,     /* All weights 1 */
   WSelectMaximal,      /* Maximal symbols in precedence get weight 0 */ 
   WSelectArity1,       /* Give weight 0 to symbols of arity 1 -> Not
			   necessarily a reduction ordering, even if,
			   usually not ground total!  */
   WCondSelectArity1,   /* Weight 0 for a unique symbol with arity 1 */
   WSelectFirstArity1,  /* Give weight 0 to first symbol with arity 1 */
   WSelectFreqArity1,   /* Give weigth 0 to a symbol with arity 1
			   that occurs most often in the axioms */
   WArityWeight,        /* Weight(f) = Arity(f)+1 */
   WAritySqWeight,      /* Weight(f) = Arity(f)^2+1) */
   WInvArityWeight,     /* Weight(f) = Maxarity+1-Arity(f) */
   WInvAritySqWeight,   /* Weight(f) = Maxarity^2+1-Arity(f)^2 */
   WPrecedence,         /* Weight(f) = |{g|g<f}| */
   WPrecedenceInv       /* Weight(f) = |{g|g>f}| */
}TOWeightGenMethod;

/* Think about goal-directedness, prefer symbols occuring in the goal */ 

#define WConstNoSpecialWeight -1


/* Default weight for symbols not treated in a special way, also used
   as multiplier for other generated weights. */

#define W_DEFAULT_WEIGHT 1


/*---------------------------------------------------------------------*/
/*                Exported Functions and Variables                     */
/*---------------------------------------------------------------------*/


extern char* TOWeightGenNames[];

#define TOGetWeightGenName(method) \
        (assert((method)>=WNoMethod), assert((method)<=WPrecedenceInv), \
	 TOWeightGenNames[(method)])

TOWeightGenMethod TOTranslateWeightGenMethod(char* name);

void TOGenerateWeights(OCB_p ocb, ClauseSet_p axioms,
		       TOWeightGenMethod method, long const_weight); 



#endif

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/






