/*-----------------------------------------------------------------------

File  : cte_termcellstore.h

Author: Stephan Schulz

Contents
 
  Abstract interface for storing term cells, implemented by a
  combination of a hashed array and term cell trees. Use
  (term->f_code^term->args[1])&TERM_STORE_HASH_MASK  as
  hash if args != NULL, otherwise use
  term->f_code&TERM_STORE_HASH_MASK.

Changes

<1> Mon Oct  5 01:09:50 MEST 1998
    New

-----------------------------------------------------------------------*/

#ifndef CTE_TERMCELLSTORE

#define CTE_TERMCELLSTORE

#include <cte_termtrees.h>

/*---------------------------------------------------------------------*/
/*                    Data type declarations                           */
/*---------------------------------------------------------------------*/

#define TERM_STORE_HASH_SIZE (4096*4)
#define TERM_STORE_HASH_MASK (TERM_STORE_HASH_SIZE-1)

typedef Term_p TermCellStore[TERM_STORE_HASH_SIZE];


/*---------------------------------------------------------------------*/
/*                Exported Functions and Variables                     */
/*---------------------------------------------------------------------*/

#define tcs_arity0hash(term) ((term)->f_code)
#define tcs_arity1hash(term) (tcs_arity0hash(term)^(((int)(term)->args[0])>>3))
#define tcs_aritynhash(term) (tcs_arity1hash(term)^(((int)(term)->args[1])>>4))

#define TermCellHash(term)\
((((term)->arity == 0)?\
  tcs_arity0hash(term):\
  (((term)->arity == 1)?tcs_arity1hash(term):tcs_aritynhash(term)))\
   &TERM_STORE_HASH_MASK)
   

void    TermCellStoreInit(TermCellStore store);
void    TermCellStoreExit(TermCellStore store);
#define TermCellStoreFind(store, term, prop_mask)\
        TermTreeFind((store)[TermCellHash(term)], term, prop_mask)
#define TermCellStoreInsert(store, term, prop_mask)\
        TermTreeInsert(&((store)[TermCellHash(term)]), term, prop_mask)
#define TermCellStoreExtract(store, term, prop_mask)\
        TermTreeExtract(&((store)[TermCellHash(term)]), term, prop_mask)
#define TermCellStoreDelete(store, term, prop_mask)\
        TermTreeDelete((&(store)[TermCellHash(term)]), term, prop_mask)
void    TermCellStoreSetProp(TermCellStore store, TermProperties
			     props); 
void    TermCellStoreDelProp(TermCellStore store, TermProperties
			     props); 
long    TermCellStoreNodes(TermCellStore store);
void    TermCellStorePrintDistrib(FILE* out, TermCellStore store);

#endif

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/





